//
// hello.txrx.45.c
//
// step response transmit-receive hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld
// 11/6/11
//
// (c) Massachusetts Institute of Technology 2011
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//
// Changes added by Linda Wanders
// FabAcademy2014 - final project development 
// 06/01/14
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin

#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define bit_delay_time 100 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define settle_delay() _delay_us(100) // settle delay
#define char_delay() _delay_ms(1) // char delay

#define nloop 50 // loops to accumulate

#define serial_port PORTB
#define serial_direction DDRB
#define serial_pin_out (1 << PB2)
#define serial_pin_in (1<<PB0)
#define serial_pins PINB

#define serial_interrupt (1 << PCIE)
#define serial_interrupt_pin (1 << PCINT0)

#define transmit_port PORTB
#define transmit_direction DDRB
#define transmit_pin (1 << PB1)

#define node_id '1'

uint16_t measuring ();

uint16_t sensor1_filtered, sensor2_filtered;

volatile char chr; //globale variabeles

void send_result ();
void send_framing ();


void get_char(volatile unsigned char *pins, unsigned char pin, volatile char *rxbyte) {
   
   // read character into rxbyte on pins pin
   // assumes line driver (inverts bits)
   
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      
	  
      // wait for start bit
      ;
   
   
   // delay to middle of first data bit
   half_bit_delay();
   bit_delay();


   // unrolled loop to read data bits
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   
   
   // wait for stop bit
   bit_delay();
   half_bit_delay();
   }
   
void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }
   
   ISR(PCINT0_vect) {
   
   // pin change interrupt handler
   
       get_char(&serial_pins, serial_pin_in, &chr);
      // put_char(&serial_port, serial_pin_out, chr);
	  
	  // clear interrupt flag  ( which was set by the pin changing during the interrupt )
	  GIFR = (1<<PCIF); 
   }

int main(void) {
   
   
   // main (initializing)
 
 
   // set clock divider to /1
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   
   
   // initialize output pins
   set(serial_port, serial_pin_out);
   set(serial_port, serial_pin_in);
   output(serial_direction, serial_pin_out);
   clear(transmit_port, transmit_pin);
   output(transmit_direction, transmit_pin);
   
   
   // setup pin change interrupt on input pin
   set(GIMSK, serial_interrupt);
   set(PCMSK, serial_interrupt_pin);
   
   
   // init A/D
   ADCSRA = (1 << ADEN) // enable
      | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
   
   
   chr=0; // initialize variable
   sei();  // set enable interupts
   
   
   // main loop
   while (1) {
		
		//set adc to PB4
		ADMUX = (0 << REFS2) | (0 << REFS1) | (0 << REFS0) // Vcc ref
		| (0 << ADLAR) // right adjust
		| (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (0 << MUX0); // PB4 (sense) 
		
		
		// fuction measuring value
		uint16_t sensor1_raw = measuring();
		
		
		if (sensor1_raw < 5000) {
			sensor1_filtered = (3*sensor1_filtered + sensor1_raw) / 4; // calculate filtering valvue
		}
		
		//if (chr==node_id) { // only send result if commanded by master board
		//  send_result ();
		//}
		
		//chr=0; //back to staring value (clear flag)
		
		
		
		
		// set adc to PB3
		ADMUX = (0 << REFS2) | (0 << REFS1) | (0 << REFS0) // Vcc ref
		| (0 << ADLAR) // right adjust
		| (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (1 << MUX0); // PB3 (sense)
		
		
		// measuring value
		uint16_t sensor2_raw = measuring();
		
		
		if (sensor1_raw < 5000) {
			sensor2_filtered = (3*sensor2_filtered + sensor2_raw) / 4; // calculate filtering valvue
		}
		
		
		if (chr==node_id) { // only send result if commanded by master board
		  chr=0; //back to staring value (clear flag)
		  send_framing ();
		  send_result ();
		}
		
		// send_framing ();
		// send_result ();
		
		}
     }

  
  uint16_t measuring () { 
	  
	  static uint16_t up,down;
	  static unsigned char count;
	  
	  
      // accumulate
      up = 0;
      down = 0;
	  
	  
      for (count = 0; count < nloop; ++count) { 
         
         // settle, charge
		 settle_delay();
         cli(); //clear interupt 
		 set(transmit_port, transmit_pin);
         
		 
         // initiate conversion
         ADCSRA |= (1 << ADSC);
         sei(); // set enable interupt
		 
		 
         // wait for completion
         while (ADCSRA & (1 << ADSC))
            ;
			
			
         // save result
         up += ADC;
         
		 
         // settle, discharge
         settle_delay();
         clear(transmit_port, transmit_pin);
         
		 
         // initiate conversion
         ADCSRA |= (1 << ADSC);
         
		 
         // wait for completion
         while (ADCSRA & (1 << ADSC))
            ;
         
		 
         // save result
         down += ADC;
         }
		
	 return (up - down);	 
 }
	
  void send_framing () {
		
		put_char(&serial_port, serial_pin_out, 1);
		char_delay();
		put_char(&serial_port, serial_pin_out, 2);
		char_delay();
		put_char(&serial_port, serial_pin_out, 3);
		char_delay();
		put_char(&serial_port, serial_pin_out, 4);
		char_delay();
	  }
	
  void send_result () {
		
		put_char(&serial_port, serial_pin_out, (sensor1_filtered & 255));
		char_delay();
		
		put_char(&serial_port, serial_pin_out, ((sensor1_filtered >> 8) & 255));
		char_delay();
		
		put_char(&serial_port, serial_pin_out, (sensor2_filtered & 255));
		char_delay();
		
		put_char(&serial_port, serial_pin_out, ((sensor2_filtered >> 8) & 255));
	  }
	
 