#
# hello.txrx.45.py
#
# receive and display transmit-receive step response
# hello.step.45.py serial_port
#
# Neil Gershenfeld
# CBA MIT 11/06/11
#
# (c) Massachusetts Institute of Technology 2011
# Permission granted for experimental and personal use;
# license for commercial sale available from MIT
#
# Changes added by Linda Wanders
# FabAcademy2014 - final project development 
# 06/01/14
#

from Tkinter import *
import serial

WINDOW = 600 # window size
eps = 0.75 # filter fraction
filter1 = 0.0 # filtered value
filter2 = 0.0 # filtered value

def idle(parent,canvas):
   global filter1, filter2, eps
   
   
   # idle routine
   byte2 = 0
   byte3 = 0
   byte4 = 0
   ser.flush()
   
   
   # find framing 
   while 1:
      byte1 = byte2
      byte2 = byte3
      byte3 = byte4
      byte4 = ord(ser.read())
      if ((byte1 == 1) & (byte2 == 2) & (byte3 == 3) & (byte4 == 4)):
         break
   
   
   # read and plot
   S1_low = ord(ser.read())
   S1_high = ord(ser.read())
   
   S2_low = ord(ser.read())
   S2_high = ord(ser.read())
   
   
   # rebuilding value
   S1_value = 256*S1_high + S1_low          
   S2_value = 256*S2_high + S2_low
   
   
   # calculating value moved to C code.
   # value = (up_value - down_value)
   
   # calculating averages moved to C code.
   # filter1 = (1-eps)*filter1 + eps*value
   # filter2 = (1-eps)*filter2 + eps*value
   
   
   x1 = int(.2*WINDOW + (.9-.2)*WINDOW*S1_value/10000.0)
   x2 = int(.2*WINDOW + (.9-.2)*WINDOW*S2_value/10000.0)
   
   canvas.itemconfigure("text1",text="%.1f"%S1_value)
   canvas.itemconfigure("text2",text="%.1f"%S2_value) 
   
   canvas.coords('rect1',.2*WINDOW,.05*WINDOW,x1,.1*WINDOW)
   canvas.coords('rect2',x1,.05*WINDOW,.9*WINDOW,.1*WINDOW)
   
   canvas.coords('rect3',.2*WINDOW,.15*WINDOW,x2,.20*WINDOW) 
   canvas.coords('rect4',x2,.15*WINDOW,.9*WINDOW,.20*WINDOW)
   
   canvas.update()
   parent.after_idle(idle,parent,canvas)


#  check command line arguments
if (len(sys.argv) != 2):
   print "command line: hello.txrx.45.py serial_port"
   sys.exit()
port = sys.argv[1]


# open serial port
ser = serial.Serial(port,9600)
ser.setDTR()


# set up GUI
root = Tk()
root.title('hello.txrx.45.py (q to exit)')
root.bind('q','exit')
canvas = Canvas(root, width=WINDOW, height=.25*WINDOW, background='white')

canvas.create_text(.1*WINDOW,.075*WINDOW,text="1",font=("Helvetica", 24),tags="text1",fill="#2c2c2c")
canvas.create_rectangle(.2*WINDOW,.05*WINDOW,.3*WINDOW,.1*WINDOW, tags='rect1', fill='#b00000')
canvas.create_rectangle(.3*WINDOW,.05*WINDOW,.9*WINDOW,.1*WINDOW, tags='rect2', fill='#eaf718')

canvas.create_text(.1*WINDOW,.175*WINDOW,text="2",font=("Helvetica", 24),tags="text2",fill="#2c2c2c")
canvas.create_rectangle(.2*WINDOW,.15*WINDOW,.3*WINDOW,.20*WINDOW, tags='rect3', fill='#002363')
canvas.create_rectangle(.3*WINDOW,.15*WINDOW,.9*WINDOW,.20*WINDOW, tags='rect4', fill='#36bf00')

canvas.pack()


# start idle loop
root.after(100,idle,root,canvas)
root.mainloop()
